import QtQuick 2.0
import Widgets 0.1
import "../Components"
import "../Components/Math.js" as MathLocal

Showable {
    id: applications

    width: units.dp(320)
    height: units.dp(854/1.5)

    property ListModel applicationsModel
    property ListModel runningModel: ListModel {}

    property bool open: false
    property bool dragging: false
    property int startValue: applications.width
    property int endValue: 0
    property int hintValue: 20
    property real progress
    property int openingDirection: Qt.LeftToRight
    property int leftMargin: 0
    property bool backgroundImageIfEmpty: true

    progress: {
        var elasticValue = elastic(__invertedprogressValue, width/4)
        if (runningModel.count == 0) {
            return 0
        } else if (runningModel.count == 1) {
            return (startValue - (width - elasticValue)) / startValue
        } else {
            return (startValue - __progressValue) / (startValue)
        }
    }

    property int __progressValue
    property int __invertedprogressValue: width - __progressValue

    Component.onCompleted: {
        if (applications.open) __progressValue = endValue
        else __progressValue = startValue

        //__createRunningModel()
        __updateLoaders()
    }

    function elastic( currentPosition, limit ) {
        return limit * ( 1 - Math.pow( ( limit - 1 ) / limit, currentPosition ) )
    }

    function activateApplication(application) {
        var found = false
        var i
        for (i = 0; i < runningModel.count; i++ ) {
            if (runningModel.get(i).identifier == application) {
                found = true
                // current application
                if (i == 0 && shown) return
                // opened to first
                runningModel.move(i,0,1)
                break;
            }
        }
        if (!found) {
            for (var i = 0; i < applicationsModel.count; i++ ) {
                if (applicationsModel.get(i).identifier == application) {
                    runningModel.insert(0, applicationsModel.get(i))
                    found = true
                }
            }
        }
        if (found) {
            animatedApplication.setApplication(runningModel.get(0))
            openNewAnimation.start()
        } else {
            //console.log("Application \"" + application + "\" not found!")
        }
    }

    function printRunningModel () {
        var i
        for (i = 0; i < runningModel.count; i++) {
            console.log(runningModel.get(i).name)
        }
        console.log("-------------")
    }

    function __createRunningModel() {
        var index
        for(index = 0; index < 4; index++) {
            runningModel.insert(index, applicationsModel.get(index))
        }
        printRunningModel()
    }

    function close() {
        __close()
        closingAnimation.start()
    }

    function __clearMainLoader() {
        mainLoader.resetApplication()
    }

    function __clearLoaders() {
        mainLoader.resetApplication()
        secondaryLoader.resetApplication()
    }

    function __showImmediately() {
        var storedDuration = showAnimation.duration
        showAnimation.duration = 0
        show()
        showAnimation.duration = storedDuration
    }

    function __updateLoaders() {
        if (runningModel.count > 0)
            mainLoader.setApplication(runningModel.get(0))
        if (runningModel.count > 1)
            secondaryLoader.setApplication(runningModel.get(1))
    }

    function __firstToLast() {
        runningModel.move(0,runningModel.count-1,1)
    }

    Rectangle {
        id: background
        width: parent.width
        height: parent.height
        color: "black"
        opacity: 0.8
    }

    ImageMipmap {
        id: backgroundImage
        anchors.fill: background
        resource: "graphics/common/empty_state.png"
        opacity: runningModel.count == 0 && backgroundImageIfEmpty ? 0.1 : 0
        Behavior on opacity { NumberAnimation {}}
    }

    Item {
        id: clippingContainer
        width: parent.width - leftMargin
        height: parent.height
        clip: true
        anchors.right: parent.right

        Loader {
            id: secondaryLoader

            property variant data

            function setApplication(data) {
                secondaryLoader.data = data
                source = "" // to force the reload
                if (data.constructor == "") {
                    source = "ApplicationWindow.qml"
                } else {
                    source = data.constructor
                }
            }

            onLoaded: {
                item.model = data
            }

            function resetApplication() {
                source = ""
            }
            width: parent.width
            height: parent.height
            scale: 0.85 + applications.progress*0.15
            opacity:  MathLocal.clamp(3*applications.progress, 0, 1)

        }

        Loader {
            id: mainLoader

            property variant data

            function setApplication(data) {
                mainLoader.data = data
                source = "" // to force the reload
                if (data.constructor == "") {
                    source = "ApplicationWindow.qml"
                } else {
                    source = data.constructor
                }
            }

            function resetApplication() {
                source = ""
            }

            onLoaded: {
                item.model = data
            }

            width: parent.width
            height: parent.height
            x: -applications.progress*width
        }
    }

    SmoothedAnimation {
        id: hintingAnimation
        target: applications
        property: "__progressValue"
        duration: 150
        to: startValue - hintValue
    }

    SequentialAnimation {
        id: openingAnimation
        NumberAnimation {
            target: applications
            property: "__progressValue"
            duration: 350
            to: endValue
            easing.type: Easing.OutCubic
        }
        ScriptAction {
            script: {
                __clearMainLoader()
                var duration
                duration = closingAnimation.duration
                closingAnimation.duration = 0
                applications.close()
                closingAnimation.duration = duration
                __firstToLast()
                __updateLoaders()
            }
        }
    }

    NumberAnimation {
        id: closingAnimation
        target: applications
        property: "__progressValue"
        duration: 350
        to: startValue
        easing.type: Easing.OutCubic
    }

    Loader {
        id: animatedApplication

        property variant data

        function setApplication(data) {
            animatedApplication.data = data
            source = "" // to force the reload
            if (data.constructor == "") {
                source = "ApplicationWindow.qml"
            } else {
                source = data.constructor
            }
        }

        onLoaded: {
            item.model = data
        }

        function resetApplication() {
            source = ""
        }

        width: parent.width - leftMargin
        height: parent.height
        visible: false
    }

    SequentialAnimation {
        id: openNewAnimation

        ScriptAction {
            script: {
                if (!shown) {
                    __clearLoaders()
                    __showImmediately()
                    background.x = applications.openingDirection == Qt.LeftToRight ? -applications.width : applications.width
                }

                animatedApplication.x = applications.openingDirection == Qt.LeftToRight ? -applications.width : applications.width
                animatedApplication.visible = true
            }
        }
        PauseAnimation {duration:  100}
        ParallelAnimation {
            NumberAnimation {
                target: animatedApplication
                properties: "x"
                to: leftMargin
                duration: 500
                easing.type: Easing.OutQuart
            }
            NumberAnimation {
                target: mainLoader
                properties: "scale"
                to: 0.8
                duration: 500
                easing.type: Easing.OutQuart
            }
            NumberAnimation {
                target: mainLoader
                properties: "opacity"
                to: 0
                duration: 400
                easing.type: Easing.OutQuad
            }
            NumberAnimation {
                target: background
                properties: "x"
                to: 0
                duration: 500
                easing.type: Easing.OutQuart
            }
        }
        ScriptAction {
            script: {
                mainLoader.opacity = 1
                mainLoader.scale = 1.0
                __updateLoaders()
                animatedApplication.visible = false
            }
        }
    }

    function __open() {
        hintingAnimation.stop()
        applications.open = true
    }

    function __close() {
        hintingAnimation.stop()
        applications.open = false
    }

    function __hint() {
        openingAnimation.stop()
        closingAnimation.stop()
        hintingAnimation.restart()
    }

    function __settle() {
        hintingAnimation.stop()
        openingAnimation.stop()
        closingAnimation.stop()
        if (open) openingAnimation.start()
        else closingAnimation.start()
    }

    function __startDragging() {
        hintingAnimation.stop()
        dragging = true
    }

    function __endDragging(dragVelocity) {
        dragging = false
        if (runningModel.count > 1 && Math.abs(dragVelocity) >= 20) {
            if (dragVelocity < 0)__open()
            else __close()
        } else {
            __settle()
        }
    }

    function __computeValue() {
        return applications.width + openDraggingArea.dragValue - hintValue
    }

    Binding {
        id: dragBinding

        target: applications
        property: "__progressValue"
        value: __computeValue()
        when: dragging
    }

    DraggingArea {
        id: openDraggingArea

        enabled: runningModel.count > 0 && !closingAnimation.running && !openingAnimation.running && !openingAnimation.running
        height: parent.height
        width: units.dp(20/1.5)
        orientation: Qt.Horizontal
        anchors.bottom: parent.bottom
        anchors.right: parent.right

        onPressed: __hint()
        onReleased: __settle()
        onDragStart: __startDragging()
        onDragEnd: __endDragging(dragVelocity)
    }
}
