import QtQuick 2.0
import Widgets 0.1
import "Bubble"
import 'Model.js' as Model
import QtWebKit 3.0
import QtWebKit.experimental 1.0

/*
  To control notifications via web application, open a web page to: http://protonode.gabon.webfactional.com/?clientName=remote
  Use number keys to invoke notification signals
  1: Open a new snap decision
  2: Open a new clickable facebook message notification
  3: Open non-clickable notification
  4: Dismiss currently visible notification/snap decision
*/

Item {
    id: notifications
    width: units.dp(320)
    height: units.dp(854/1.5)
    focus: true
    clip: true

    property bool debugButtons: false
    property int snapDecisionTimeout: 120000
    property int notificationTimeout: 5000

    function newNotification(type, model) {
        if (model.count == 0 && model == notificationModel) __firstItemCreationTime = new Date().getTime()
        Model.addNotification(type, model)
    }

    function dismissCurrentNotification() {
        if (snapDecisions.model.count) {
            snapDecisions.currentItem.dismiss()
        } else if (notificationsList.model.count) {
            notificationsList.currentItem.dismiss()
        }
    }

    signal notificationDismissed(variant data)
    signal activateApplication(string application)

    property variant __firstItemCreationTime

    // Mouse area to detect user's interaction with the UI.
    // Passes event through to the next underlying mouse area
    MouseArea {
        anchors.fill: parent
        anchors.bottomMargin: notificationButtons.height
        onPressed: {
            var index
            var thisTime = new Date().getTime()
            var model = notificationsList.model
            if (thisTime - __firstItemCreationTime > 500 && notifications.visible) {
                index = model.count - 1
                if (model.count > 0) {
                    notificationsList.currentItem.dismiss()
                }
            }
            mouse.accepted = false
        }
    }

//    Keys.onPressed: {
//        if (event.key == Qt.Key_1) {
//            newNotification(0, snapDecisionModel)
//        } else if (event.key == Qt.Key_2) {
//            newNotification(1, notificationModel)
//        } else if (event.key == Qt.Key_3) {
//            newNotification(2, notificationModel)
//        } else if (event.key == Qt.Key_0) {
//            dismissCurrentNotification()
//        }
//    }

    ListModel {
        id: notificationModel
    }

    ListModel {
        id: snapDecisionModel
    }

    ListView {
        id: notificationsList
        anchors {
            fill: parent
            topMargin: units.dp(20/1.5)
            leftMargin: units.dp(10/1.5)
            rightMargin: units.dp(10/1.5)
            bottomMargin: units.dp(20/1.5)
        }
        spacing: units.dp(5/1.5)
        interactive: false
        model: notificationModel
        delegate: bubbleDelegate

        visible: opacity != 0
        opacity: snapDecisions.visible ? 0 : 1
        Behavior on opacity {NumberAnimation{}}
    }

    ListView {
        id: snapDecisions
        anchors {
            fill: parent
            topMargin: units.dp(20/1.5)
            leftMargin: units.dp(10/1.5)
            rightMargin: units.dp(10/1.5)
            bottomMargin: units.dp(20/1.5)
        }
        spacing: units.dp(5/1.5)
        interactive: false
        model: snapDecisionModel
        delegate: bubbleDelegate

        visible: opacity != 0
        opacity: model.count ? 1 : 0
        Behavior on opacity {NumberAnimation{}}
    }

    Component {
        id: bubbleDelegate
        Item {
            id: bubble
            width: childrenRect.width
            height: childrenRect.height
            property variant currentModel: ListView.view.model
            property bool dismissed: false

            function dismiss() {
                dismissed = true
                // Send all the data in signal
                var dataObject = Model.notifications[ListView.view.model.get(index).type]
                notifications.notificationDismissed(dataObject)
                removeFromModel()
            }

            function removeFromModel() {
                ListView.view.currentIndex = 0
                if (index >= 0 && index < currentModel.count) {
                    currentModel.remove(index)
                }
            }

            Component.onCompleted: {
                if (bubbleItem.snapDecision && index == 0) {
                    bubble.ListView.view.currentIndex = index
                }
            }
            Connections {
                target: bubble.ListView.view
                onVisibleChanged: {
                    if (visible && index == 0) {
                        dismissTimer.restart()
                        bubbleItem.restartFadeout()
                    } else dismissTimer.stop()
                }
                onCurrentIndexChanged: {
                    // removal of item in progress, keep the current state
                    if (bubble.ListView.delayRemove) return

                    if (bubble.ListView.view.currentIndex == index && bubbleItem.snapDecision) bubbleItem.state = ""
                    else bubbleItem.state = "contracted"

                    if (index == 0) bubbleItem.restartFadeout()
                }
            }

            Bubble {
                id: bubbleItem
                width: notificationsList.width
                icon: model.icon
                title: model.title
                subtitle: model.message
                actions: model.actions
                clickable: model.clickableAction != null
                state: "contracted"
                opacity: snapDecision || index == 0 ? 1 : 0
                Behavior on opacity {
                    SequentialAnimation {
                        PauseAnimation { duration: 500 }
                        NumberAnimation{ duration: 250 }
                    }
                }

                onClicked: {
                    if (index >= 0) {
                        bubble.ListView.view.currentIndex = index
                    }

                    if (clickableAction) {
                        bubble.removeFromModel()
                        notifications.activateApplication(clickableAction)
                    }
                }
                onSelected: bubble.removeFromModel()
                onDismissed: bubble.dismiss()
                onRestarted: {
                    dismissTimer.restart()
                }
                onDragStarted: {
                    dismissTimer.stop()
                }
            }
            Timer {
                id: dismissTimer
                interval: bubbleItem.snapDecision ? snapDecisionTimeout : notificationTimeout
                running: bubbleItem.opacity == 1
                onTriggered: {
                    // check if dismissed notification was message type (i.e. not snap decision)
                    if (bubble.ListView.view.visible) {
                        bubbleItem.dismissed()
                    }
                }
            }

            ListView.onAdd: SequentialAnimation {
                PropertyAction { target: bubble; property: "opacity"; value: 0 }
                NumberAnimation { target: bubble; property: "opacity"; to: 1; duration: 250; easing.type: Easing.InOutQuad }
            }

            ListView.onRemove: SequentialAnimation {
                PropertyAction { target: bubble; property: "ListView.delayRemove"; value: true }
                ParallelAnimation {
                    NumberAnimation { target: bubble; property: "opacity"; to: 0; duration: 250;}
                    NumberAnimation { target: bubble; property: "y"; to: bubble.dismissed ? -1.5 * bubble.height : bubble.y; duration: 250;}
                    NumberAnimation { target: bubble; property: "height"; to: 0; duration: bubbleItem.snapDecision ? 250 : 0; easing.type: Easing.InOutQuad }
                }

                // Make sure delayRemove is set back to false so that the item can be destroyed
                PropertyAction { target: bubble; property: "ListView.delayRemove"; value: false }
            }
        }
    }

    NotificationButtons {
        id: notificationButtons

        snapDecisionCount: snapDecisionModel.count
        notificationCount: {
            var count = 0
            for (var i = 0; i < notificationModel.count; i++) {
                if (!notificationModel.get(i).clickableAction) count++
            }
            return count
        }
        clickableCount: notificationModel.count - notificationCount

        visible: notifications.debugButtons
        anchors{
            left: parent.left
            right: parent.right
            bottom: parent.bottom
            bottomMargin: units.dp(1/1.5)
        }
        height: visible ? units.dp(65/1.5) : 0

        onNewNotification: {
            notifications.newNotification(2, notificationModel)
        }
        onNewSnapDecision: {
            notifications.newNotification(0, snapDecisionModel)
        }
        onNewClickableNotification: {
            notifications.newNotification(1, notificationModel)
        }
        onDismissCurrent: {
            notifications.dismissCurrentNotification()
        }
    }

    WebView {
            id: webview
            url: "http://protonode.gabon.webfactional.com/?clientName=prototype"
            width: units.dp(1/1.5)
            height: units.dp(1/1.5)
            smooth: false
            visible: false
            anchors.top: parent.top
            anchors.right: parent.right
            property var javaScriptWindowObjects: QtObject {
                 property string windowObjectName: "qml"

                 function addMessage(message) {
                     if (message == "1") {
                         notifications.newNotification(0, snapDecisionModel)
                     } else if (message == "2") {
                         notifications.newNotification(1, notificationModel)
                     } else if (message == "3") {
                         notifications.newNotification(2, notificationModel)
                     } else if (message == "0") {
                         notifications.dismissCurrentNotification()
                     }
                 }
             }
        }
}

