import QtQuick 2.0
import Widgets 0.1
import "Dash"
import "Applications"
import "Greeter"
import "Launcher"
import "Indicators"
import "Notifications"
import "OnScreenKeyboard"
import "Hud"
import "SideStage"
import "Components"
import "Components/Math.js" as MathLocal

Item {
    id: shell

    width: tablet ? units.dp(1280) : units.dp(320)
    height: tablet ? units.dp(800) : units.dp(854/1.5)

    property bool tablet: false

    property ListModel phoneApplicationsModel: ApplicationsModel {}
    property ListModel tabletApplicationsModel: TabletApplicationsModel {}
    property ListModel applicationsModel: tablet ? tabletApplicationsModel : phoneApplicationsModel

    property ListModel sideStageApplicationsModel: sideStageFilter.filteredModel
    property variant sideStageFilter: FilterModel {
        sourceModel: applicationsModel
        function isItemAccepted(item) {
            return item.sideStage
        }
    }

    property ListModel mainStageApplicationsModel: mainStageFilter.filteredModel
    property variant mainStageFilter: FilterModel {
        sourceModel: applicationsModel
        function isItemAccepted(item) {
            return !item.sideStage
        }
    }

    Component.onCompleted: addLauncherFavourites()

    function addLauncherFavourites() {
        // add apps
        for (var i = 0; i < applicationsModel.count; i++) {
            if ( applicationsModel.get(i).favourite ) {
                launcher.add(applicationsModel.get(i))
            }
        }
    }

    function activateApplication(desktopFile) {
        applications.activateApplication(desktopFile)
        sideStage.activateApplication(desktopFile)
    }

    function activateDash() {
        dash.show()
    }

    function showOnScreenKeyboard() {
        onScreenKeyboard.show()
    }

    Units {
        id: units
    }

    ImageMipmap {
        id: bg
        resource: tablet ? "graphics/tablet_background.jpg" : "graphics/phone_background.jpg"
        anchors.fill: parent
    }

    Dash {
        id: dash

        available: !greeter.shown
        hides: [applications, launcher, indicators]
        shown: true

        anchors {
            top: parent.top
            topMargin: indicators.topbarHeight
            bottom: parent.bottom
        }
        width: parent.width
        scale: 1 - 0.2*applicationsRevealer.animatedProgress
        spreadModel: applications.runningModel
        clip: scale != 1
        opacity: {
            if (applications.runningModel.count == 0) {
                return 1 - applicationsRevealer.animatedProgress
            } else {
                if (applicationsRevealer.animatedProgress < 0.99) {
                    return 1
                } else {
                    return 0
                }
            }
        }
        Behavior on opacity {enabled: applications.runningModel.count != 0; StandardAnimation{}}

        onApplicationSelected: activateApplication(identifier)
    }

    Item {

        width: parent.width
        height: parent.height
        x: launcher.progress
        Behavior on x {SmoothedAnimation{velocity: 600}}


        Applications {
            id: applications

            property bool sideStagePreviouslyShown: false
            available: !greeter.shown
            hides: [launcher, indicators]
            shown: false
            onShownChanged: {
                if (!shown) {
                    sideStagePreviouslyShown = sideStage.shown
                    sideStage.hide()
                } else {
                    if (sideStagePreviouslyShown) sideStage.show()
                }
            }
            opacity: 1.0
            showAnimation: StandardAnimation { property: "x"; duration: 350; to: applicationsRevealer.openedValue; easing.type: Easing.OutCubic }
            hideAnimation: StandardAnimation { property: "x"; duration: 350; to: applicationsRevealer.closedValue; easing.type: Easing.OutCubic }

            y: indicators.topbarHeight
            width: parent.width
            height: parent.height - indicators.topbarHeight - indicators.separatorLineHeight
            applicationsModel: shell.mainStageApplicationsModel
        }
    }


    Revealer {
        id: applicationsRevealer

        property real animatedProgress: MathLocal.clamp((-dragPosition - launcher.progress) / closedValue, 0, 1)
        Behavior on animatedProgress {enabled: !applicationsRevealer.dragging; NumberAnimation{duration: 450; easing.type: Easing.OutCubic}}
        enabled: applications.runningModel.count > 0
        direction: Qt.RightToLeft
        openedValue: 0
        hintDisplacement: units.dp(20)
        closedValue: width
        target: applications
        width: applications.width
        height: applications.height
        orientation: Qt.Horizontal
    }

    SideStage {
        id: sideStage

        available: !greeter.shown
        openingDirection: Qt.RightToLeft
        shown: false
        onShownChanged: {
            if (shown) {
                applications.show()
            }
        }
        opacity: 1.0
        showAnimation: StandardAnimation { property: "x"; duration: 350; to: sideStageRevealer.openedValue; easing.type: Easing.OutCubic }
        hideAnimation: StandardAnimation { property: "x"; duration: 350; to: sideStageRevealer.closedValue; easing.type: Easing.OutCubic }

        visible: tablet
        y: indicators.topbarHeight
        width: tablet ? units.dp(320) + leftMargin : units.dp(0)
        height: parent.height - indicators.topbarHeight - indicators.separatorLineHeight
        applicationsModel: shell.sideStageApplicationsModel
        leftMargin: units.dp(10)
        handleVisible: sideStageRevealer.draggingArea.pressed
    }

    Revealer {
        id: sideStageRevealer

        visible: !(applications.runningModel.count != 0 && !applications.shown) && tablet
        direction: Qt.RightToLeft
        openedValue: shell.width - sideStage.width
        closedValue: shell.width
        hintDisplacement: units.dp(20)
        handleSize: units.dp(35)
        target: sideStage
        width: sideStage.width + units.dp(12)
        height: sideStage.height
        anchors.top: sideStage.top
        orientation: Qt.Horizontal
        anchors.right: parent.right

        function dragToValue(dragPosition) {
            var offset = target.shown ? hintDisplacement : 0
            return dragPosition + closedValue + offset
        }
    }


    SidestageHandle {
        id: handle
        anchors {
            top: sideStage.top
            bottom: sideStage.bottom
            left: sideStage.left
            leftMargin: -12
        }
        width: units.dp(34)
        opacity: sideStage.handleVisible ? 0.3 : 0
        Behavior on opacity { NumberAnimation { duration: 70 } }
    }

    Greeter {
        id: greeter

        available: true
        hides: [launcher, indicators]
        shown: true
        showAnimation: StandardAnimation { property: "x"; to: greeterRevealer.openedValue }
        hideAnimation: StandardAnimation { property: "x"; to: greeterRevealer.closedValue }

        y: indicators.topbarHeight
        width: parent.width
        height: parent.height - indicators.topbarHeight
    }

    Revealer {
        id: greeterRevealer

        target: greeter
        width: greeter.width
        height: greeter.height
        orientation: Qt.Horizontal
        visible: greeter.shown
    }

    Item {
        id: overlay

        anchors.fill: parent

        Launcher {
            id: launcher

            anchors.top: parent.top
            anchors.topMargin: indicators.topbarHeight
            anchors.bottom: parent.bottom
            width: parent.width
            applicationFocused: applications.shown
            shortcutsWidth: units.dp(120/1.5)
            iconPath: "graphics/applicationIcons"
            onDashItemSelected: {
                // Animate if moving between application and dash
                if (!applications.shown) {
                    dash.focusTo("home", true)
                } else {
                    dash.focusTo("home", false)
                }
                applications.hide()
            }
            onDash: {
                applications.hide()
            }
            onLauncherApplicationSelected:{
                shell.activateApplication(name)
            }
            onStateChanged: {
                if (state == "spreadMoving") {
                    dash.focusTo("application", false)
                }
            }
        }

        Indicators {
            id: indicators

            width: parent.width
            height: parent.height
            rowWidth: tablet ? units.dp(320) : parent.width
            available: !greeter.shown
            hides: []
            shown: false
            revealer: indicatorRevealer
            hintValue: indicatorRevealer.hintDisplacement
            topbarHeight: units.dp(24)
            showAnimation: StandardAnimation { property: "progress"; duration: 350; to: indicatorRevealer.openedValue; easing.type: Easing.OutCubic }
            hideAnimation: StandardAnimation { property: "progress"; duration: 350; to: indicatorRevealer.closedValue; easing.type: Easing.OutCubic }
            openedHeight: parent.height

            onStateChanged: {
                if (!tablet) {
                    if (state == "reveal" || state == "hint") {
                        hud.hideDown()
                    } else if (state == "initial") {
                        hud.show()
                    } else if (state == "locked" || state == "commit") {
                        hud.hideUp()
                    }
                }
            }
        }

        Revealer {
            id: indicatorRevealer

            width: indicators.rowWidth
            anchors.right: parent.right
            height: parent.height
            target: indicators
            handleSize: indicators.topbarHeight
            closedValue: indicators.topbarHeight + indicators.separatorLineHeight
            openedValue: indicators.openedHeight
            dragVelocityThreshold: units.dp(1/1.5)
            boundProperty: "progress"
            hintDisplacement: 3*indicators.topbarHeight
            orientation: Qt.Vertical

            onOpenClicked: {
                indicators.openOverview()
                indicators.show()
            }
            onCloseClicked: {
                indicators.hide()
            }

            onOpenPressed: indicators.handlePress()

            function dragToValue(dragPosition) {
                var offset = target.shown ? 0 : -hintDisplacement + handleSize
                return dragPosition + offset
            }
        }

        Hud {
            id: hud

            width: units.dp(100)
            height: indicators.topbarHeight
            anchors {
                top: indicators.top
                left: indicators.left
            }
        }

        LauncherLabel {
            id: launcherLabel

            property int margin: units.dp(10)

            x: launcher.shortcutsWidth + margin
            y: launcher.labelPosition + launcher.y - height / 2
            width: shell.tablet ? shell.width / 4 : shell.width - launcher.shortcutsWidth - 2 * margin
            height: launcher.labelHeight
            text: launcher.currentItemText
            iconSource: launcher.currentIconSource
            opacity: launcher.labelVisible ? 1 : 0

            Behavior on opacity {
                enabled: launcherLabel.opacity < 1.0
                SequentialAnimation {
                    PauseAnimation { duration: 100 }
                    NumberAnimation {duration: 100}
                }
            }
        }

        Notifications {
            id: notifications

            width: parent.width
            height: parent.height
            y: indicators.shown ? 0 : indicators.topbarHeight + indicators.separatorLineHeight
            onNotificationDismissed: {
                indicators.newMessageData(data)
            }
            onActivateApplication: shell.activateApplication(application)
        }
    }

    focus: true
    OnScreenKeyboard {
        id: onScreenKeyboard

        // FIXME: should be visible when a TextInput has the focus
        shown: false
        y: parent.height
        showAnimation: StandardAnimation { property: "y"; to: onScreenKeyboard.parent.height - onScreenKeyboard.height }
        hideAnimation: StandardAnimation { property: "y"; to: onScreenKeyboard.parent.height }

        width: parent.width
    }

    Item {
        id: quitButton
        width: units.dp(100/1.5)
        height: units.dp(50/1.5)
        MouseArea {
            anchors.fill: parent
            onPressAndHold: quitResetDialog.show()
        }
    }

    QuitResetDialog {
        id: quitResetDialog
        anchors.fill: parent
    }
}
